'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Node = exports.Connections = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _requestPromise = require('request-promise');

var rp = _interopRequireWildcard(_requestPromise);

var _immutable = require('immutable');

require('./state.js');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Connections = exports.Connections = function () {
  function Connections(nodes, badNodes) {
    _classCallCheck(this, Connections);

    this.knownNodes = (0, _immutable.List)();
    this.knownBadNodes = (0, _immutable.List)();
    if (nodes != undefined) this.knownNodes = (0, _immutable.List)(nodes);
    if (badNodes != undefined) this.knownBadNodes = (0, _immutable.List)(badNodes);
  }

  _createClass(Connections, [{
    key: 'randomNode',
    value: function randomNode() {
      return; // Need to pull random from this.knownNodes and return!
    }
  }, {
    key: 'addNode',
    value: function addNode(host, port) {
      if (port != undefined) {
        this.knownNodes = this.knownNodes.push(new Node(host, port));
      } else {
        this.knownNodes = this.knownNodes.push(host);
      }
    }
  }]);

  return Connections;
}();

var Node = exports.Node = function () {
  function Node(host, port) {
    _classCallCheck(this, Node);

    this.host = host;
    this.port = Number(port);

    this.authToken = null;
    this.apiVersion = 'v1';
  }

  _createClass(Node, [{
    key: 'get',
    value: function get(endpoint, qs) {
      if (qs === undefined) {
        qs = {};
      }

      return rp({
        url: 'http://' + this.host + ':' + this.port + '/api/' + this.apiVersion + '/' + endpoint,
        qs: qs,
        method: 'GET'
      });
    }
  }, {
    key: 'post',

    // Pass formData in as a JSON object of key value pairs.
    value: function post(endpoint, formData) {
      return rp({
        url: 'http://' + this.host + ':' + this.port + '/api/' + this.apiVersion + '/' + endpoint,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        form: formData
      });
    }
  }, {
    key: 'status',


    // Get status of a particular node.
    value: function status() {
      return this.get('status');
    }
  }, {
    key: 'authenticate',


    // Authenticates a node with a credential you pass in.
    value: function authenticate(credential) {
      var node = this;
      return this.get('auth').then(function (response) {
        signature = credential.signHex(JSON.parse(response).challenge)[0];
        formData = {
          'challenge': signature
        };
        return node.post('auth', formData).then(function (response) {
          var authToken = JSON.parse(response).authToken;
          node.authToken = authToken;
          return authToken;
        });
      });
    }
  }, {
    key: 'getState',
    value: function getState() {
      return this.get('state');
    }
  }, {
    key: 'searchState',
    value: function searchState(type, address) {
      return this.getState().then(State.searchState(response, type, address));
    }
  }, {
    key: 'searchTransactions',
    value: function searchTransactions(query_params) {
      return this.get('transaction', query_params);
    }
  }, {
    key: 'getTransaction',
    value: function getTransaction(txhash) {
      return this.get('transaction/' + txhash);
    }
  }, {
    key: 'submitTransaction',
    value: function submitTransaction(tx_params) {
      return this.post('transaction', tx_params);
    }
  }]);

  return Node;
}();